#include "stdafx.h"
#include "NeMessenger.h"
#include "StickerList.h"
#include "ScrollHelper.h"

#define SL_THUMBNAIL_WIDTH		62
#define SL_THUMBNAIL_HEIGHT		70

IMPLEMENT_DYNAMIC(CStickerList, CWnd)

CStickerList::CStickerList()
{
	m_pVecAllSticker = NULL;
	m_vecCurrentSticker.clear();
	m_strSelectedGroupID = _T("");
	m_nMouseOverStickerIndex = -1;
	m_pSticketSelectBg = NULL;
	m_bLbuttonDown = false;
	m_pChatWnd = NULL;

	m_scrollHelper = new CScrollHelper;
	m_scrollHelper->AttachWnd(this);
}

CStickerList::~CStickerList()
{
	SAFE_DELETE(m_pSticketSelectBg);
	delete m_scrollHelper;
}

BEGIN_MESSAGE_MAP(CStickerList, CWnd)
	ON_WM_CREATE()
	ON_WM_PAINT()
	ON_WM_SIZE()
	ON_WM_VSCROLL()
	ON_WM_MOUSEWHEEL()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSELEAVE()
	ON_WM_LBUTTONUP()
END_MESSAGE_MAP()

int CStickerList::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_pSticketSelectBg = new Image(GetAppImgFile(_T("Chat_sticker_select_bg.png")));

	if (!m_pSticketSelectBg)
		return -1;
	
	return 0;
}

void CStickerList::ResizeItems()
{
	if (!m_pVecAllSticker)
		return;

	CRect rtClient;
	GetClientRect(&rtClient);

	const int gapX = 17;
	const int gapY = 17;
	const int startX = 10;

	int nSX = startX;
	int nSY = 10;
	int nEY = nSY;	//  Ʒ Y
	const float fThumbnailRatio = (float)SL_THUMBNAIL_HEIGHT / (float)SL_THUMBNAIL_WIDTH;

	m_vecCurrentSticker.clear();

	for (DWORD i = 0; i < m_pVecAllSticker->size(); i++)
	{
		CStickerItem* pItem = (CStickerItem*)m_pVecAllSticker->at(i);

		if (!pItem)
			continue;

		if (pItem->m_strGroupID.CompareNoCase(m_strSelectedGroupID) != 0)
			continue;

		if (nSX + SL_THUMBNAIL_WIDTH >= rtClient.right)	//  ο ׸   ٷ
		{
			nSX = startX;
			nSY += SL_THUMBNAIL_HEIGHT + gapY;
		}

		// ü  ũ
		pItem->m_rtItem.left = nSX;
		pItem->m_rtItem.top = nSY;
		pItem->m_rtItem.right = pItem->m_rtItem.left + SL_THUMBNAIL_WIDTH;
		pItem->m_rtItem.bottom = pItem->m_rtItem.top + SL_THUMBNAIL_HEIGHT;

		pItem->m_rtThumbnail = pItem->m_rtItem;

		// ̹  ( ä  ϱ )
		if (pItem->m_img)
		{
			UINT nImgHeight = pItem->m_img->GetHeight();
			UINT nImgWidth = pItem->m_img->GetWidth();

			float fImgRatio = (float)nImgHeight / (float)nImgWidth;

			if (fImgRatio > fThumbnailRatio)	// ̰   ̹. ̴   ʺ ٿ Ѵ.
			{
				float fHeightRatio = (float)SL_THUMBNAIL_HEIGHT / (float)nImgHeight;
				UINT nThumbWidth = (UINT)(nImgWidth * fHeightRatio);
				UINT nOffsetX = (SL_THUMBNAIL_WIDTH - nThumbWidth) / 2;

				pItem->m_rtThumbnail.left = pItem->m_rtThumbnail.left + nOffsetX;
				pItem->m_rtThumbnail.right = pItem->m_rtThumbnail.left + nThumbWidth;
			}
			else	
			{
				float fWidthRatio = (float)SL_THUMBNAIL_WIDTH / (float)nImgWidth;
				UINT nThumbHeight = (UINT)(nImgHeight * fWidthRatio);
				UINT nOffsetY = (SL_THUMBNAIL_HEIGHT - nThumbHeight) / 2;

				pItem->m_rtThumbnail.top = pItem->m_rtThumbnail.top + nOffsetY;
				pItem->m_rtThumbnail.bottom = pItem->m_rtThumbnail.top + nThumbHeight;
			}
		}

		nSX += SL_THUMBNAIL_WIDTH + gapX;

		if (pItem->m_rtItem.bottom > nEY)
			nEY = pItem->m_rtItem.bottom;

		m_vecCurrentSticker.push_back(pItem);
	}

	m_scrollHelper->SetDisplaySize(rtClient.Width(), nEY + 12);
}

void CStickerList::OnPaint()
{
	CPaintDC dc(this); // device context for painting

	PAINTSTRUCT ps = dc.m_ps;
	int psWidth = ps.rcPaint.right - ps.rcPaint.left;
	int psHeight = ps.rcPaint.bottom - ps.rcPaint.top;

	int nPX = ps.rcPaint.left;
	int nPY = ps.rcPaint.top;

	CRect rtPS(nPX, nPY, nPX + psWidth, nPY + psHeight);

	CRect rt;
	GetClientRect(&rt);
	int nWndWidth = rt.Width();
	int nWndHeight = rt.Height();

	HDC hdc = dc.GetSafeHdc();

	HDC	hdcMemory = CreateCompatibleDC(hdc);

	if (hdcMemory == NULL)
		return;

	HBITMAP hbmpMem = CreateCompatibleBitmap(hdc, psWidth, psHeight);

	HGDIOBJ hbmpOld = ::SelectObject(hdcMemory,  hbmpMem);

	Graphics graph(hdcMemory);

	if (hbmpMem)
	{
		graph.SetPageUnit(UnitPixel);
		graph.SetInterpolationMode(InterpolationModeHighQuality);
		ImageAttributes ImgAttr;

		graph.FillRectangle(&SolidBrush(Color(255,255,255,255)), 0, 0, psWidth, psHeight);

		for (DWORD i = 0; i < m_vecCurrentSticker.size(); i++)
		{
			CStickerItem* pItem = (CStickerItem*)m_vecCurrentSticker[i];

			if (!pItem || !pItem->m_img)
				continue;

			//  ̹  (̹ 4̴)
			CRect rtBgSrc;
			rtBgSrc.left = 0;

			if (m_pSticketSelectBg && i == m_nMouseOverStickerIndex)	//  콺  
			{
				if (m_bLbuttonDown)
					rtBgSrc.left = (m_pSticketSelectBg->GetWidth() / 4) * 2;
				else
					rtBgSrc.left = (m_pSticketSelectBg->GetWidth() / 4) * 1;
			}
			
			if (m_pSticketSelectBg)
			{
				rtBgSrc.right = rtBgSrc.left + m_pSticketSelectBg->GetWidth() / 4;
				rtBgSrc.top = 0;
				rtBgSrc.bottom = rtBgSrc.top + m_pSticketSelectBg->GetHeight();

				CRect rtBGDest(pItem->m_rtItem.left - 4, pItem->m_rtItem.top - 4, pItem->m_rtItem.left - 4 + m_pSticketSelectBg->GetWidth() / 4, pItem->m_rtItem.top - 4 + m_pSticketSelectBg->GetHeight());
				rtBGDest.OffsetRect(0, -m_scrollHelper->GetScrollPos().cy);
				DrawImageToDest(&graph, m_pSticketSelectBg, &rtPS, &rtBGDest, &rtBgSrc, &ImgAttr);	//  

				CRect rtThumbnail = pItem->m_rtThumbnail;
				rtThumbnail.OffsetRect(0, -m_scrollHelper->GetScrollPos().cy);

				DrawImageToDest(&graph, pItem->m_img, &rtPS, &rtThumbnail, &ImgAttr);	// ƼĿ 
			}
		}

		BitBlt(dc.GetSafeHdc(), nPX, nPY, psWidth, psHeight, hdcMemory, 0, 0, SRCCOPY);

		::SelectObject(hdcMemory, hbmpOld);
		::DeleteObject(hbmpMem);
	}

	::DeleteDC(hdcMemory);
}

void CStickerList::ShowSticker(CString strGroupID)
{
	m_strSelectedGroupID = strGroupID;
	m_nMouseOverStickerIndex = -1;

	ResizeItems();
	RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
}


void CStickerList::OnSize(UINT nType, int cx, int cy)
{
	CWnd::OnSize(nType, cx, cy);
	m_scrollHelper->OnSize(nType, cx, cy);

	ResizeItems();
}

void CStickerList::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	m_scrollHelper->OnVScroll(nSBCode, nPos, pScrollBar);
}

BOOL CStickerList::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
	BOOL wasScrolled = m_scrollHelper->OnMouseWheel(nFlags, zDelta, pt);

	return wasScrolled;
}

void CStickerList::OnMouseMove(UINT nFlags, CPoint point)
{
	int nMouseOverItem = CheckPointInSticker(point);

	if (nMouseOverItem != m_nMouseOverStickerIndex)
	{
		m_nMouseOverStickerIndex = nMouseOverItem;
		RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
	}

	CWnd::OnMouseMove(nFlags, point);
}

void CStickerList::OnLButtonDown(UINT nFlags, CPoint point)
{
	if (m_nMouseOverStickerIndex > -1)
	{
		m_bLbuttonDown = true;
		RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);

		CStickerItem* pItem = (CStickerItem*)m_vecCurrentSticker[m_nMouseOverStickerIndex];

		if (!pItem)
			return;

		if (m_pChatWnd)
			m_pChatWnd->PostMessage(WM_CHAT_STICKER_SELECTED, (WPARAM)&pItem->m_strStickerID, NULL);
	}

	CWnd::OnLButtonDown(nFlags, point);
}

void CStickerList::OnLButtonUp(UINT nFlags, CPoint point)
{
	m_bLbuttonDown = false;
	RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);

	CWnd::OnLButtonUp(nFlags, point);
}


void CStickerList::OnMouseLeave()
{
	m_nMouseOverStickerIndex = -1;
	RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);

	CWnd::OnMouseLeave();
}

int CStickerList::CheckPointInSticker(CPoint point)
{
	int nIndex = -1;

	for (DWORD i = 0; i < m_vecCurrentSticker.size(); i++)
	{
		CStickerItem* pItem = (CStickerItem*)m_vecCurrentSticker[i];

		if (!pItem)
			continue;

		CRect rtItemScrollPos = pItem->m_rtItem;
		rtItemScrollPos.OffsetRect(0, -m_scrollHelper->GetScrollPos().cy);

		if (rtItemScrollPos.PtInRect(point))
		{
			nIndex = i;
			break;
		}
	}

	return nIndex;
}

void CStickerList::AllocStickerDataPointer(std::vector<CStickerItem*>* pVecAllSticker)
{
	//  ƼĿ   ִ  ͸ Ҵ.
	// ޸ Ҵ   ȭâ ο ϹǷ ⼭ ޾Ƽ 븸 ϸ ȴ.
	m_pVecAllSticker = pVecAllSticker;
}

void CStickerList::SetParentChatWnd( CWnd* pParent )
{
	m_pChatWnd = pParent;
}

